// Copyright 2014 Google Inc.

#import <Foundation/Foundation.h>

#import "GCKGameplayState.h"
#import "GCKLobbyState.h"
#import "GCKPlayerInfo.h"
#import "GCKPlayerState.h"

/**
 * @file GCKGameManagerState.h
 */

/**
 * Represents a snapshot of the state of the Game Manager running on the receiver device.
 *
 * @ingroup Games
 */
GCK_EXPORT
@interface GCKGameManagerState : NSObject

/**
 * The lobby state. Valid values: {@link GCKLobbyState}
 */
@property(nonatomic, readonly) GCKLobbyState lobbyState;

/**
 * The gameplay state. Valid values: {@link GCKGameplayState}
 */
@property(nonatomic, readonly) GCKGameplayState gameplayState;

/**
 * The game specific data. Will be either an object that can be serialized to JSON using
 * NSJSONSerialization, or nil.
 */
@property(nonatomic, copy, readonly) id gameData;

/**
 * The game status text.
 */
@property(nonatomic, copy, readonly) NSString *gameStatusText;

/**
 * The list of all players in the game.
 */
@property(nonatomic, readonly) NSArray *players;

/**
 * The list of all the players created by this sender device.
 */
@property(nonatomic, readonly) NSArray *controllablePlayers;

/**
 * The list of players in a connected state. A player is considered to be in a connected state if
 * the associated {@link GCKPlayerInfo#playerState} is one of
 * {@link GCKPlayerStateAvailable}, {@link GCKPlayerStateReady},
 * {@link GCKPlayerStateIdle} or {@link GCKPlayerStatePlaying}.
 */
@property(nonatomic, readonly) NSArray *connectedPlayers;

/**
 * The list of players in a connected state that were also created on this sender device. A player
 * is considered to be in a connected state if the associated {@link GCKPlayerInfo#playerState} is
 * one of {@link GCKPlayerStateAvailable}, {@link GCKPlayerStateReady}, {@link GCKPlayerStateIdle}
 * or {@link GCKPlayerStatePlaying}.
 */
@property(nonatomic, readonly) NSArray *connectedControllablePlayers;

/**
 * The application name. Returns nil if {@link GCKGameManagerChannel} is not yet connected to the
 * receiver.
 */
@property(nonatomic, copy, readonly) NSString *applicationName;

/**
 * The maximum number of players as defined by the receiver. Returns 0 if
 * {@link GCKGameManagerChannel} is not yet connected to the receiver.
 */
@property(nonatomic, readonly) NSInteger maxPlayers;

/**
 * Returns the playerInfo for the specified playerID or null if the player does not
 * exist.
 *
 * @param playerID The id of the player.
 */
- (GCKPlayerInfo *)getPlayer:(NSString *)playerID;

/**
 * Returns a list of players that are in the specified player state.
 *
 * @param playerState The {@link GCKPlayerState} each player has to be in.
 */
- (NSArray *)getPlayersInState:(GCKPlayerState)playerState;

/**
 * Returns whether the lobby state is different between this object and the specified
 * {@link GCKGameManagerState}.
 *
 * @param otherState
 */
- (BOOL)hasLobbyStateChanged:(GCKGameManagerState *)otherState;

/**
 * Returns whether the gameplay state is different between this object and the specified
 * {@link GCKGameManagerState}.
 */
- (BOOL)hasGameplayStateChanged:(GCKGameManagerState *)otherState;

/**
 * Returns whether the game data is different between this object and the specified
 * {@link GCKGameManagerState}.
 */
- (BOOL)hasGameDataChanged:(GCKGameManagerState *)otherState;

/**
 * Returns whether game status text is different between this object and the specified
 * {@link GCKGameManagerState}.
 */
- (BOOL)hasGameStatusTextChanged:(GCKGameManagerState *)otherState;

/**
 * Returns whether the player with the specified player ID has changed between this object and the
 * specified {@link GCKGameManagerState}.
 */
- (BOOL)hasPlayerChanged:(NSString *)playerId otherState:(GCKGameManagerState *)otherState;

/**
 * Returns whether the player state of the player with the specified player ID has changed between
 * this object and the specified {@link GCKGameManagerState}.
 */
- (BOOL)hasPlayerStateChanged:(NSString *)playerId otherState:(GCKGameManagerState *)otherState;

/**
 * Returns whether the player data of the player with the specified player ID has changed between
 * this object and the specified {@link GCKGameManagerState}.
 */
- (BOOL)hasPlayerDataChanged:(NSString *)playerId otherState:(GCKGameManagerState *)otherState;

/**
 * Returns a list of player IDs that are different between this object and the specified
 * {@link GCKGameManagerState}. This includes players that were added, removed, or have changed in
 * any way.
 */
- (NSArray *)getListOfChangedPlayers:(GCKGameManagerState *)otherState;

@end
